'use strict'


/**
 * Modules
 * Node
 * @constant
 */
const path = require('path')

/**
 * Modules
 * External
 * @constant
 */
const appRootPath = require('app-root-path')
appRootPath.setPath(path.join(__dirname, '..'))
const CliProgress = require('cli-progress')
const colors = require('colors/safe')
const moment = require('moment')
/* eslint-disable no-unused-vars */
const momentDurationFormat = require('moment-duration-format')
/* eslint-enable */


/**
 * Modules
 * Internal
 * @constant
 */
const utils = require(path.join(appRootPath.path, 'lib', 'utils'))

/**
 * Timecode format
 * @constant
 * @default
 */
const etaFormat = 'HH:mm:ss'

/**
 * Convert seconds into timecode format
 * @param {Number} seconds - Duration in seconds
 * @return {String} - Timecode / HH:mm:ss
 */
let convertSecondsToTimecode = (seconds) => {
    // console.debug('convertSecondsToTimecode')

    return moment.duration(seconds, 'seconds').format(etaFormat, { trim: false })
}

/**
 * Progress Bar Singleton
 * @external {CliProgress.Bar}
 */
let progressBar

/**
 * Wrapper for progressBar constructor
 * @param {Object=} options - Bar configuration options
 * @return {CliProgress.Bar} - progressBar
 */
let createBar = (options) => {
    // console.debug('createProgressbar')

    const configuration = {
        format: `🎬 ${colors.grey('Rendering')} ${colors.green.bold('{filename}')} ${colors.white.bold('|')} ${colors.blue('{bar}')}  ${colors.blue.bold('{percentagePad}%')} ${colors.white.bold('|')} ${colors.grey('ETA')} ${colors.red.bold('{etaTimecode}')}`,
        fps: 20,
        barsize: 60,
        barCompleteChar: '\u2588',
        barIncompleteChar: '\u2591',
        etaBuffer: 64,
        hideCursor: true,
        stream: process.stdout,
        position: 'center',
        stopOnComplete: true,
        clearOnComplete: false
    }

    return new CliProgress.Bar(Object.assign(configuration, options), CliProgress.Presets.shades_grey)
}

/**
 * Wrapper for progressBar.start
 * @param {Number} total - Bar total value
 * @param {String} filename - Bar filename
 * @param {Number} width - Bar width
 * @public
 */
let startBar = (total, filename, width) => {
    // console.debug('startProgressbar')

    progressBar.start(total, 0, {
        etaTimecode: convertSecondsToTimecode(0),
        fractionToPercentagePadded: '00',
        filename: filename
    })
}

/**
 *  Wrapper for progressBar.update
 * @param {Number} currentValue - Progress Bar current value
 * @public
 */

let updateBar = (currentValue) => {
    // console.debug('updateProgressbar')

    progressBar.update(currentValue, {
        etaTimecode: convertSecondsToTimecode(progressBar.eta.eta),
        percentagePad: utils.fractionToPercentagePad(progressBar.value / progressBar.total)
    })
}

/**
 * @param {Object} options - Progress Bar options
 * @exports progress-bar
 * @returns {Object}
 */
module.exports = (options) => {
    progressBar = createBar(options)

    return {
        bar: progressBar,
        start: startBar,
        update: updateBar
    }
}
